package com.btcpiyush.ads.google_applovin_unity_ads;

import android.util.Log;

import androidx.annotation.NonNull;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxError;
import com.google.android.gms.ads.AdError;
import com.google.android.gms.ads.LoadAdError;

import java.lang.ref.WeakReference;

 public class DelegatingApplovinInterstitialAdLoadCallback
        implements MaxAdListener {

    private final WeakReference<FlutterAppLovinInterstitialAd> delegate;

    public DelegatingApplovinInterstitialAdLoadCallback(FlutterAppLovinInterstitialAd delegate) {
        this.delegate = new WeakReference<>(delegate);
    }

    @Override
    public void onAdLoaded(MaxAd ad) {
        Log.e( "onAdLoaded: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().onAdLoaded(ad);
        }
    }

    @Override
    public void onAdDisplayed(MaxAd ad) {
        Log.e( "onAdDisplayed: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().manager.onAdShowedFullScreenContent(delegate.get().adId);
        }
    }

    @Override
    public void onAdHidden(MaxAd ad) {
        Log.e( "onAdHidden: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().manager.onAdDismissedFullScreenContent(delegate.get().adId);
        }
    }

    @Override
    public void onAdClicked(MaxAd ad) {
        Log.e( "onAdClicked: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().manager.onAdClicked(delegate.get().adId);
        }
    }

    @Override
    public void onAdLoadFailed(String adUnitId, MaxError error) {
        Log.e( "onAdLoadFailed: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().onAdFailedToLoad(new LoadAdError(error.getCode(), error.getMessage(), String.valueOf(error.getMediatedNetworkErrorCode()), null, null));
        }
    }

    @Override
    public void onAdDisplayFailed(MaxAd ad, MaxError error) {
        Log.e( "onAdDisplayFailed: ", "DelegatingApplovinInterstitialAdLoadCallback");
        if (delegate.get() != null) {
            delegate.get().manager.onFailedToShowFullScreenContent(delegate.get().adId, new AdError(error.getCode(), error.getMessage(), String.valueOf(error.getMediatedNetworkErrorCode())));
        }
    }
}
