package com.btcpiyush.ads.google_applovin_unity_ads_example

import android.app.Activity
import android.util.Log
import android.widget.Toast
import com.appharbr.sdk.configuration.AHSdkConfiguration
import com.appharbr.sdk.engine.*
import com.appharbr.sdk.engine.adformat.AdFormat
import com.appharbr.sdk.engine.listeners.AHListener
import com.appharbr.sdk.engine.listeners.OnAppHarbrInitializationCompleteListener
import com.applovin.mediation.MaxAdListener
import com.applovin.mediation.MaxRewardedAdListener
import com.applovin.mediation.ads.MaxAdView
import com.applovin.mediation.ads.MaxInterstitialAd
import com.applovin.mediation.ads.MaxRewardedAd
import com.applovin.sdk.AppLovinSdk
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin.GeoedgeSdkFactory
import io.flutter.embedding.android.FlutterActivity
import java.util.*

data class CustomGeoSdk(val activity: FlutterActivity) : GeoedgeSdkFactory {

    override fun initializeGeoSdk(geoedgeSdkKey: String?) {
        if (geoedgeSdkKey.isNullOrEmpty()) {
            return@initializeGeoSdk
        }

        val ahSdkConfiguration = AHSdkConfiguration.Builder(geoedgeSdkKey)
            .withMuteAd(true)
            .build()

        AppHarbr.initialize(activity.applicationContext,
            ahSdkConfiguration,
            object : OnAppHarbrInitializationCompleteListener {
                override fun onSuccess() {
                    Toast.makeText(
                        activity,
                        "AppHarbr SDK Initialized Successfully",
                        Toast.LENGTH_SHORT
                    ).show()
                    Log.e("LOG", "AppHarbr SDK Initialized Successfully")
                }

                override fun onFailure(reason: InitializationFailureReason) {
                    Toast.makeText(
                        activity,
                        "AppHarbr SDK Initialized onFailure",
                        Toast.LENGTH_SHORT
                    ).show()
                    Log.e(
                        "LOG",
                        "AppHarbr SDK Initialization Failed: " + reason.readableHumanReason
                    )
                }
            })
    }

    override fun addBannerAd(maxAdView: MaxAdView?) {
        val ahListener =
            AHListener { view: Any?, unitId: String?, adFormat: AdFormat?, reasons: Array<AdBlockReason?>? ->
                Log.e(
                    "LOG",
                    "AppHarbr - onAdBlocked for: $unitId, AppHarbr Banner reason: " + Arrays.toString(
                        reasons
                    )
                )
            }

        maxAdView?.let {
            AppHarbr.addBannerView(AdSdk.MAX, maxAdView, activity.lifecycle, ahListener)
        }

    }

    override fun addInterstitialAdGeoSdk(
        mInterstitialAppLovinAd: MaxInterstitialAd,
        mInterstitialAppLovinCallback: MaxAdListener
    ) {
        val ahListener =
            AHListener { view: Any?, unitId: String?, adFormat: AdFormat?, reasons: Array<AdBlockReason?>? ->
                /*Toast.makeText(act, "AppHarbr - onAdBlocked for: $unitId, reason: " + Arrays.toString(
                    reasons
                ), Toast.LENGTH_SHORT ).show()
                Log.e(
                    "LOG",
                    "AppHarbr - onAdBlocked for: $unitId, reason: " + Arrays.toString(
                        reasons
                    )
                )*/
            }

        val callback = AppHarbr.addInterstitial(
            AdSdk.MAX,
            mInterstitialAppLovinAd,
            mInterstitialAppLovinCallback,
            activity.lifecycle,
            ahListener
        )
        mInterstitialAppLovinAd.setListener(callback)
        mInterstitialAppLovinAd.loadAd()
    }

    override fun loadAppLovinInterstitial(
        adUnitId: String,
        _appLovinSDK: AppLovinSdk,
        mInterstitialAppLovinCallback: MaxAdListener
    ): MaxInterstitialAd {

        val adview = MaxInterstitialAd(
            adUnitId, _appLovinSDK, activity
        )

        val ahListener =
            AHListener { view: Any?, unitId: String?, adFormat: AdFormat?, reasons: Array<AdBlockReason?>? ->

                Toast.makeText(
                    activity,
                    "AppHarbr - onAdBlocked for: $unitId, reason: " + Arrays.toString(
                        reasons
                    ),
                    Toast.LENGTH_SHORT
                ).show()

                Log.e(
                    "LOG",
                    "AppHarbr - onAdBlocked for: $unitId, reason: " + Arrays.toString(
                        reasons
                    )
                )

            }

        val ahWrapperListener: MaxAdListener? = AppHarbr.addInterstitial(
            AdSdk.MAX,
            adview,
            mInterstitialAppLovinCallback,
            activity.lifecycle,
            ahListener
        )

        // Set ahWrapperListener and load Ad
        adview.setListener(ahWrapperListener)
        adview.loadAd()
        return adview
    }

    override fun isGeoAdShow(mInterstitialAppLovinAd: MaxInterstitialAd): Boolean {
        Toast.makeText(activity, "AppHarbr SDK isGeoAdShow", Toast.LENGTH_SHORT).show()
        if (!mInterstitialAppLovinAd.isReady) {
            return false
        }
        return AppHarbr.getInterstitialState(mInterstitialAppLovinAd) != AdStateResult.BLOCKED
    }

}