import 'dart:async';

import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:get/get.dart';
import 'package:google_applovin_unity_ads/ads/SettingsModel.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:flutter/widgets.dart';

const AdRequest _request = AdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

const AdManagerAdRequest _ad_manager_request = AdManagerAdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

BannerAd? _bannerAd;
NativeAd? _nativeAd;
AppLovinNativeAd? _appLovinnativeAd;
FacebookNativeAd? _facebooknativeAd;
AdManagerBannerAd? _adManagerBannerAd;
AppLovinBannerAd? _appLovinBannerAd;
int _TOTALDELAYD = 0;

const String _GOOGLEAD = "google_ads";
const String _ADMANAGERAD = "ad_manager";
const String _APPLOVINAD = "app_lovin";
const String _FACEBOOKAD = "facebook_ads";
const String _CUSTOMAD = "custom_ads";

Settings? _settings = getAdsSettings();
int appVersionCode = getAppVersionCode();

void setTimerDelayed() {
  _TOTALDELAYD = 1500;
}

void resetTimerDelayed() {
  _TOTALDELAYD = 0;
}

void destroyAd() {
  getInterestitialAd()?.dispose();
  getAdManagerInterstitialAd()?.dispose();
  getRewardedAd()?.dispose();
  _adManagerBannerAd?.dispose();
  _bannerAd?.dispose();
  _appLovinBannerAd?.dispose();
  _appLovinnativeAd?.dispose();
  _nativeAd?.dispose();
}

initOpenAds({Function? onOpenAdLoaded, Function? onOpenAdFailedLoaded}) {
  if (_settings == null) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  AppOpenAdManager appOpenAdManager = AppOpenAdManager()
    ..loadAd(callback: onOpenAdLoaded);

  setAppLifecycleReactor(
      AppLifecycleReactor(appOpenAdManager: appOpenAdManager));

  getAppLifecycleReactor()?.listenToAppStateChanges();
}

showOpenAds() {
  if (_settings == null) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  getAppLifecycleReactor()?.appOpenAdManager.showAdIfAvailable();
}

showIntraAds({Function? callback, bool isCompulsory = false}) async {
  showProgressbarDialog(Get.context!);
  loadIntraAds(
      isCompulsory: isCompulsory,
      onAdLoadedCallback: () => {
            Future.delayed(Duration(milliseconds: _TOTALDELAYD),
                () => {Get.back(), _showInterstitialAd(callback)})
          });
}

showRewardAds(
    {Function? callback,
    Function? onFailedCallback,
    Function? onSkippedCallback}) async {
  showProgressbarDialog(Get.context!);
  resetTimerDelayed();
  loadRewardAds(
      onAdLoadedCallback: () => {
            Future.delayed(
                Duration(milliseconds: _TOTALDELAYD),
                () => {
                      Get.back(),
                      _showRewardedAd(
                          callback, onFailedCallback, onSkippedCallback)
                    })
          },
      onAdFailedCallback: () => {
            Future.delayed(
                Duration(milliseconds: _TOTALDELAYD),
                () => {
                      Get.back(),
                      if (onFailedCallback != null) {onFailedCallback()}
                    })
          });
}

showBannerAds(
    {int pos = 0,
    AdSize size = AdSize.banner,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
      if ((_settings?.googleAds?.isBannerAds ?? false) &&
          (_settings?.googleAds?.bannerId ?? "").isNotEmpty) {
        BannerAd(
                size: size,
                adUnitId: _settings?.googleAds?.bannerId ?? "",
                listener: BannerAdListener(
                  onAdLoaded: (Ad ad) {
                    print('$ad loaded.');
                    if (onAdLoadedCallback != null && ad is AdWithView) {
                      onAdLoadedCallback(Container(
                        width: size.width.toDouble(),
                        height: size.height.toDouble(),
                        child: AdWidget(ad: ad),
                      ));
                    }
                  },
                  onAdFailedToLoad: (Ad ad, LoadAdError error) {
                    print('$BannerAd failedToLoad: $error');
                    ad.dispose();
                    showBannerAds(
                        pos: pos + 1,
                        size: size,
                        onAdLoadedCallback: onAdLoadedCallback,
                        onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                  },
                  onAdOpened: (Ad ad) => print('$BannerAd onAdOpened.'),
                  onAdClosed: (Ad ad) => print('$BannerAd onAdClosed.'),
                ),
                request: _request)
            .load();
      } else {
        showBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _ADMANAGERAD:
      if ((_settings?.googleAds?.isBannerAds ?? false) &&
          (_settings?.googleAds?.bannerId ?? "").isNotEmpty) {
        AdManagerBannerAd(
          adUnitId: _settings?.googleAds?.bannerId ?? "",
          sizes: <AdSize>[size],
          request: _ad_manager_request,
          listener: AdManagerBannerAdListener(
            onAdLoaded: (Ad ad) async {
              // print('Inline adaptive banner loaded: ${ad.responseInfo}');
              if (onAdLoadedCallback != null && ad is AdWithView) {
                onAdLoadedCallback(Container(
                  width: size.width.toDouble(),
                  height: size.height.toDouble(),
                  child: AdWidget(ad: ad),
                ));
              }
              // After the ad is loaded, get the platform ad size and use it to
              // update the height of the container. This is necessary because the
              // height can change after the ad is loaded.

              /* AdManagerBannerAd bannerAd = (ad as AdManagerBannerAd);
              final AdSize? size = await bannerAd.getPlatformAdSize();

              if (size == null) {
                print('Error: getPlatformAdSize() returned null for $bannerAd');
                return;
              }*/
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              // _adManagerBannerAd = null;
              showBannerAds(
                  pos: pos + 1,
                  size: size,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
          ),
        ).load();
      } else {
        showBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _APPLOVINAD:
      if ((_settings?.appLovin?.isBannerAds ?? false) &&
          (_settings?.appLovin?.bannerId ?? "").isNotEmpty) {
        AppLovinBannerAd(
          adUnitId: _settings?.appLovin?.bannerId ?? "",
          size: size,
          listener: BannerAdListener(
            onAdLoaded: (Ad ad) async {
              // print('Inline adaptive banner loaded: ${ad.responseInfo}');
              if (onAdLoadedCallback != null && ad is AdWithView) {
                onAdLoadedCallback(Container(
                  width: size.width.toDouble(),
                  height: size.height.toDouble(),
                  child: AdWidget(ad: ad),
                ));
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              showBannerAds(
                  pos: pos + 1,
                  size: size,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
          ),
        ).load();
      } else {
        showBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _FACEBOOKAD:
      if ((_settings?.facebookAds?.isBannerAds ?? false) &&
          (_settings?.facebookAds?.bannerId ?? "").isNotEmpty) {
        FacebookBannerAd(
          adUnitId: _settings?.facebookAds?.bannerId ?? "",
          size: size,
          listener: BannerAdListener(
            onAdLoaded: (Ad ad) async {
              // print('Inline adaptive banner loaded: ${ad.responseInfo}');
              if (onAdLoadedCallback != null && ad is AdWithView) {
                onAdLoadedCallback(Container(
                  width: size.width.toDouble(),
                  height: size.height.toDouble(),
                  child: AdWidget(ad: ad),
                ));
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              showBannerAds(
                  pos: pos + 1,
                  size: size,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
          ),
        ).load();
      } else {
        showBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    default:
      showBannerAds(
          pos: pos + 1,
          size: size,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      break;
  }
}

showNativeAds(
    {int pos = 0,
    bool isMediumNativeAds = false,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    EdgeInsetsGeometry? widgetMargin,
    double borderRadius = 12,
    Size size = const Size(double.infinity, 350),
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  switch (_settings!.adsSequence![pos]) {
    case _GOOGLEAD:
      if ((_settings?.googleAds?.isNativeAds ?? false) &&
          (_settings?.googleAds?.nativeId ?? "").isNotEmpty) {
        _nativeAd = NativeAd(
          adUnitId: _settings?.googleAds?.nativeId ?? "",
          isMediumNativeAds: isMediumNativeAds,
          request: _request,
          factoryId: 'nativeFactoryId',
          listener: NativeAdListener(
            onAdLoaded: (Ad nativeAd) {
              print('$NativeAd loaded.');
              if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                onAdLoadedCallback(Container(
                  width: size.width.toDouble(),
                  height: size.height.toDouble(),
                  margin: widgetMargin,
                  alignment: Alignment.center,
                  decoration: BoxDecoration(
                      color: backgroundColor,
                      border: Border.all(
                          color: borderColor,
                          width: borderwidth,
                          style: borderstyle),
                      borderRadius:
                          BorderRadius.all(Radius.circular(borderRadius))),
                  child: AdWidget(ad: nativeAd),
                ));
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print(
                  '$NativeAd == = = ==== ==== ==== ==== ==== ==== === failedToLoad: $error');
              ad.dispose();
              _nativeAd = null;
              showNativeAds(
                  pos: pos + 1,
                  size: size,
                  widgetMargin: widgetMargin,
                  backgroundColor: backgroundColor,
                  borderColor: borderColor,
                  borderwidth: borderwidth,
                  borderstyle: borderstyle,
                  borderRadius: borderRadius,
                  isMediumNativeAds: isMediumNativeAds,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
            onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
            onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
          ),
        )..load();
      } else {
        showNativeAds(
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _ADMANAGERAD:
      if ((_settings?.googleAds?.isNativeAds ?? false) &&
          (_settings?.googleAds?.nativeId ?? "").isNotEmpty) {
        _nativeAd = NativeAd.fromAdManagerRequest(
            adUnitId: _settings?.googleAds?.nativeId ?? "",
            isMediumNativeAds: isMediumNativeAds,
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                _nativeAd = null;
                showNativeAds(
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ),
            adManagerRequest: _ad_manager_request)
          ..load();
      } else {
        showNativeAds(
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            isMediumNativeAds: isMediumNativeAds,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _APPLOVINAD:
      if ((_settings?.appLovin?.isNativeAds ?? false) &&
          (_settings?.appLovin?.nativeAds?.manual ?? "").isNotEmpty) {
        _appLovinnativeAd = AppLovinNativeAd(
            isMediumNativeAds: isMediumNativeAds,
            adUnitId: _settings?.appLovin?.nativeAds?.manual ?? "",
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                _appLovinnativeAd = null;
                showNativeAds(
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ))
          ..load();
      } else {
        showNativeAds(
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _FACEBOOKAD:
      if ((_settings?.facebookAds?.isNativeAds ?? false) &&
          (_settings?.facebookAds?.nativeId ?? "").isNotEmpty) {
        _facebooknativeAd = FacebookNativeAd(
            isMediumNativeAds: isMediumNativeAds,
            adUnitId: _settings?.facebookAds?.nativeId ?? "",
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad nativeAd) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null && nativeAd is AdWithView) {
                  onAdLoadedCallback(Container(
                    width: size.width.toDouble(),
                    height: size.height.toDouble(),
                    alignment: Alignment.center,
                    margin: widgetMargin,
                    decoration: BoxDecoration(
                        color: backgroundColor,
                        border: Border.all(
                            color: borderColor,
                            width: borderwidth,
                            style: borderstyle),
                        borderRadius:
                            BorderRadius.all(Radius.circular(borderRadius))),
                    child: AdWidget(ad: nativeAd),
                  ));
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                _appLovinnativeAd = null;
                showNativeAds(
                    pos: pos + 1,
                    size: size,
                    widgetMargin: widgetMargin,
                    backgroundColor: backgroundColor,
                    borderColor: borderColor,
                    borderwidth: borderwidth,
                    borderstyle: borderstyle,
                    borderRadius: borderRadius,
                    isMediumNativeAds: isMediumNativeAds,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ))
          ..load();
      } else {
        showNativeAds(
            pos: pos + 1,
            size: size,
            widgetMargin: widgetMargin,
            isMediumNativeAds: isMediumNativeAds,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    default:
      showNativeAds(
          pos: pos + 1,
          size: size,
          widgetMargin: widgetMargin,
          isMediumNativeAds: isMediumNativeAds,
          backgroundColor: backgroundColor,
          borderColor: borderColor,
          borderwidth: borderwidth,
          borderstyle: borderstyle,
          borderRadius: borderRadius,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      break;
  }
}

showMediumNativeAds(
    {Size size = const Size(double.infinity, 160),
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  showNativeAds(
      pos: 0,
      size: size,
      widgetMargin: widgetMargin,
      isMediumNativeAds: true,
      backgroundColor: backgroundColor,
      borderColor: borderColor,
      borderwidth: borderwidth,
      borderstyle: borderstyle,
      borderRadius: borderRadius,
      onAdLoadedCallback: onAdLoadedCallback,
      onAdFailedToLoadCallback: onAdFailedToLoadCallback);
}

IsFullIntraAds({Function? callback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    showIntraAds(callback: callback);
    return;
  }
  if (callback != null) {
    callback();
  }
}

showFullNativeAds(
    {Size? size,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    (size != null)
        ? showNativeAds(
            pos: 0,
            size: size,
            isMediumNativeAds: true,
            widgetMargin: widgetMargin,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback)
        : showNativeAds(
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            widgetMargin: widgetMargin,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
    return;
  }

  if (onAdLoadedCallback != null) {
    onAdLoadedCallback(Container(height: 0));
  }
}

showFullMediumNativeAds(
    {Size? size,
    Color backgroundColor = const Color(0xFFFFFFFF),
    Color borderColor = const Color(0xFFFFFFFF),
    double borderwidth = 1.0,
    EdgeInsetsGeometry? widgetMargin,
    BorderStyle borderstyle = BorderStyle.solid,
    double borderRadius = 12,
    Function(Widget)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    (size != null)
        ? showMediumNativeAds(
            size: size,
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            widgetMargin: widgetMargin,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback)
        : showMediumNativeAds(
            backgroundColor: backgroundColor,
            borderColor: borderColor,
            borderwidth: borderwidth,
            widgetMargin: widgetMargin,
            borderstyle: borderstyle,
            borderRadius: borderRadius,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
    return;
  }

  if (onAdLoadedCallback != null) {
    onAdLoadedCallback(Container(height: 0));
  }
}

_setAdsinList(RxList<dynamic> list, int distance) {
  int pos = 0;
  for (var i = 0; i < list.length; i++) {
    if (pos > 0 && ((pos + 1) % distance == 0)) {
      try {
        if (list[pos] != null && !(list[pos] is Widget)) list.insert(pos, null);
      } catch (e) {
        print(e);
      }
    }
    pos++;
  }
}

showAdsInListView(
    {required int adsDistance,
    required IndexedWidgetBuilder itemBuilder,
    bool isNativeAds = true,
    Size? nativeAdSize,
    Color nativeAdbackgroundColor = const Color(0xFFFFFFFF),
    Color nativeAdborderColor = const Color(0xFFFFFFFF),
    double nativeAdborderwidth = 1.0,
    BorderStyle nativeAdborderstyle = BorderStyle.solid,
    double nativeAdborderRadius = 12,
    EdgeInsetsGeometry? widgetMargin,
    AdSize bannerSize = AdSize.leaderboard,
    required AdsController controller}) {
  _setAdsinList(controller.widgetlist, adsDistance);

  _onScrollEventStop(
      controller,
      adsDistance,
      isNativeAds,
      nativeAdSize,
      nativeAdbackgroundColor,
      nativeAdborderColor,
      nativeAdborderwidth,
      nativeAdborderstyle,
      nativeAdborderRadius,
      bannerSize,
      widgetMargin);

  return Obx(() => NotificationListener<ScrollNotification>(
        onNotification: (scrollNotification) {
          if (scrollNotification is ScrollStartNotification) {
            controller.isScroll.value = true;
          } else if (scrollNotification is ScrollUpdateNotification) {
            controller.isScroll.value = true;
          } else if (scrollNotification is ScrollEndNotification) {
            controller.isScroll.value = false;
            _onScrollEventStop(
                controller,
                adsDistance,
                isNativeAds,
                nativeAdSize,
                nativeAdbackgroundColor,
                nativeAdborderColor,
                nativeAdborderwidth,
                nativeAdborderstyle,
                nativeAdborderRadius,
                bannerSize,
                widgetMargin);
          } else {
            controller.isScroll.value = false;
            print(
                "Scroll is ------- ${controller.isScroll.value}  ${controller.currentIndex.value}");
          }
          return controller.isScroll.value;
        },
        child: ListView.separated(
            itemBuilder: itemBuilder,
            separatorBuilder: (context, index) {
              if (controller.widgetlist[index] is Widget) {
                return controller.widgetlist[index];
              }
              return Container(height: 0);
            },
            itemCount: controller.widgetlist.length),
      ));
}

_onScrollEventStop(
    AdsController controller,
    int adsDistance,
    bool isNativeAds,
    Size? nativeAdSize,
    Color nativeAdbackgroundColor,
    Color nativeAdborderColor,
    double nativeAdborderwidth,
    BorderStyle nativeAdborderstyle,
    double nativeAdborderRadius,
    AdSize bannerSize,
    EdgeInsetsGeometry? widgetMargin) {
  if (controller.widgetlist.length == 0) {
    return;
  }

  final startIndex = (controller.currentIndex.value - adsDistance) < 0
      ? 0
      : (controller.currentIndex.value - adsDistance);
  final lastIndex = (controller.currentIndex.value + adsDistance) >=
          controller.widgetlist.length
      ? (controller.widgetlist.length - 1)
      : (controller.currentIndex.value + adsDistance);

  for (var pos = startIndex; pos <= lastIndex; pos++) {
    if (controller.widgetlist[pos] == null && !controller.isScroll.value) {
      if (isNativeAds) {
        if (nativeAdSize != null) {
          showMediumNativeAds(
              size: nativeAdSize,
              backgroundColor: nativeAdbackgroundColor,
              widgetMargin: widgetMargin,
              borderColor: nativeAdborderColor,
              borderRadius: nativeAdborderRadius,
              borderstyle: nativeAdborderstyle,
              borderwidth: nativeAdborderwidth,
              onAdLoadedCallback: (nativeAd) {
                controller.widgetlist[controller.currentIndex.value] = nativeAd;
                controller.refresh();
              });
        } else {
          showMediumNativeAds(
              backgroundColor: nativeAdbackgroundColor,
              borderColor: nativeAdborderColor,
              borderRadius: nativeAdborderRadius,
              widgetMargin: widgetMargin,
              borderstyle: nativeAdborderstyle,
              borderwidth: nativeAdborderwidth,
              onAdLoadedCallback: (nativeAd) {
                controller.widgetlist[pos] = nativeAd;
                controller.refresh();
              });
        }
      } else {
        showBannerAds(
            size: bannerSize,
            onAdLoadedCallback: (bannerAd) {
              controller.widgetlist[pos] = bannerAd;
              controller.refresh();
            });
      }
    }
  }
}

showAdsInGridView(
    {required RxList<dynamic> list,
    required int crossAxisCount,
    required int adsDistance,
    required IndexedWidgetBuilder itemBuilder,
    bool isNativeAds = true,
    AdSize bannerSize = AdSize.leaderboard,
    Size? nativeAdSize,
    Color nativeAdbackgroundColor = const Color(0xFFFFFFFF),
    Color nativeAdborderColor = const Color(0xFFFFFFFF),
    double nativeAdborderwidth = 1.0,
    BorderStyle nativeAdborderstyle = BorderStyle.solid,
    double nativeAdborderRadius = 12,
    double mainAxisSpacing = 0.0,
    double crossAxisSpacing = 0.0,
    EdgeInsetsGeometry? widgetMargin,
    required GetxController controller}) {
  final distance = (adsDistance * crossAxisCount) + 1;

  _setAdsinList(list, distance);

  return Obx(() => StaggeredGridView.countBuilder(
        crossAxisCount: crossAxisCount,
        itemCount: list.length,
        itemBuilder: (BuildContext context, int index) {
          if (list[index] == null) {
            if (isNativeAds) {
              if (nativeAdSize != null) {
                showMediumNativeAds(
                    backgroundColor: nativeAdbackgroundColor,
                    borderColor: nativeAdborderColor,
                    borderRadius: nativeAdborderRadius,
                    widgetMargin: widgetMargin,
                    borderstyle: nativeAdborderstyle,
                    borderwidth: nativeAdborderwidth,
                    size: nativeAdSize,
                    onAdLoadedCallback: (nativeAd) {
                      list[index] = nativeAd;
                      controller.refresh();
                    });
              } else {
                showMediumNativeAds(
                    backgroundColor: nativeAdbackgroundColor,
                    borderColor: nativeAdborderColor,
                    borderRadius: nativeAdborderRadius,
                    borderstyle: nativeAdborderstyle,
                    borderwidth: nativeAdborderwidth,
                    onAdLoadedCallback: (nativeAd) {
                      list[index] = nativeAd;
                      controller.refresh();
                    });
              }
            } else {
              showBannerAds(
                  size: bannerSize,
                  onAdLoadedCallback: (bannerAd) {
                    list[index] = bannerAd;
                    controller.refresh();
                  });
            }
            return Container(height: 0);
          } else if (list[index] is Widget) {
            return list[index];
          }
          return itemBuilder(context, index);
        },
        staggeredTileBuilder: (int index) => StaggeredTile.count(
            ((list[index] == null) ||
                    (list[index] != null && list[index] is Widget))
                ? crossAxisCount
                : 1,
            1),
        mainAxisSpacing: mainAxisSpacing,
        crossAxisSpacing: crossAxisSpacing,
      ));
}

void loadIntraAds(
    {int pos = 0, bool isCompulsory = false, Function? onAdLoadedCallback}) {
  if (_settings == null) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    resetTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  final isInterestial = ((_settings?.adSetting?.interestialAdsInterval ?? 0) >
          0 &&
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
              (_settings?.adSetting?.interestialAdsInterval ?? 0) ==
          0 &&
      ((getInterestialAdTimeInterval() +
                  ((_settings?.adSetting?.interestialAdsTimingInterval ?? 0) *
                      1000)) <=
              DateTime.now().millisecondsSinceEpoch ||
          getInterestialAdTimeInterval() == 0));

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if (((_settings?.googleAds?.interstitialInterval ?? 0) <= 0 &&
              isInterestial) ||
          ((_settings?.googleAds?.interstitialInterval ?? 0) > 0 &&
              ((_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
                      (_settings?.googleAds?.interstitialInterval ?? 0) ==
                  0)) ||
          isCompulsory) {
        if ((_settings?.googleAds?.isInterstitialAds ?? true) &&
            (_settings?.googleAds?.interestialId ?? "").isNotEmpty) {
          if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
            _createAdManagerInterstitialAd(
                pos, isCompulsory, onAdLoadedCallback);
            return;
          }
          _createInterstitialAd(pos, isCompulsory, onAdLoadedCallback);
          return;
        }
      }
      loadIntraAds(
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    case _APPLOVINAD:
      if (isInterestial ||
          (_settings?.appLovin?.isUnlimitedAds ?? false) ||
          isCompulsory) {
        if ((_settings?.appLovin?.isInterstitialAds ?? true) &&
            (_settings?.appLovin?.interestialId ?? "").isNotEmpty) {
          _createAppLovinInterstitialAd(pos, isCompulsory, onAdLoadedCallback);
          return;
        }
      }
      loadIntraAds(
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    case _FACEBOOKAD:
      if (((_settings?.facebookAds?.interstitialInterval ?? 0) <= 0 &&
              isInterestial) ||
          ((_settings?.facebookAds?.interstitialInterval ?? 0) > 0 &&
              ((_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
                      (_settings?.facebookAds?.interstitialInterval ?? 0) ==
                  0)) ||
          isCompulsory) {
        if ((_settings?.facebookAds?.isInterstitialAds ?? true) &&
            (_settings?.facebookAds?.interestialId ?? "").isNotEmpty) {
          _createFacebookInterstitialAd(pos, isCompulsory, onAdLoadedCallback);
          return;
        }
      }
      loadIntraAds(
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
    default:
      loadIntraAds(
          pos: pos + 1,
          isCompulsory: isCompulsory,
          onAdLoadedCallback: onAdLoadedCallback);
      break;
  }
}

void loadRewardAds(
    {int pos = 0, Function? onAdLoadedCallback, Function? onAdFailedCallback}) {
  if (_settings == null) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    resetTimerDelayed();
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  print('RewardedAd pos  $pos ');
  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if ((_settings?.googleAds?.isRewaredAds ?? false) &&
          (_settings?.googleAds?.rewardedId ?? "").isNotEmpty) {
        if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
          _createAdmanagerRewardedAd(
              pos, onAdLoadedCallback, onAdFailedCallback);
          return;
        }
        _createRewardedAd(pos, onAdLoadedCallback, onAdFailedCallback);
        return;
      } else {
        loadRewardAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    case _APPLOVINAD:
      if ((_settings?.appLovin?.isRewaredAds ?? false) &&
          (_settings?.appLovin?.rewaredId ?? "").isNotEmpty) {
        _createAppLovinRewardedAd(pos, onAdLoadedCallback, onAdFailedCallback);
        return;
      } else {
        loadRewardAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    case _FACEBOOKAD:
      if ((_settings?.facebookAds?.isRewaredAds ?? false) &&
          (_settings?.facebookAds?.rewardedId ?? "").isNotEmpty) {
        _createFacebookRewardedAd(pos, onAdLoadedCallback, onAdFailedCallback);
        return;
      } else {
        loadRewardAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    default:
      loadRewardAds(
          pos: pos + 1,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedCallback: onAdFailedCallback);
      break;
  }
}

void _createInterstitialAd(
    int pos, bool isCompulsory, Function? onAdLoadedCallback) {
  if (getInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  InterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _request,
      adLoadCallback: InterstitialAdLoadCallback(
        onAdLoaded: (InterstitialAd ad) {
          print('$ad loaded');
          setInterestitialAd(ad);
          getInterestitialAd()?.setImmersiveMode(true);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setInterestitialAd(null);
          loadIntraAds(
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createAppLovinInterstitialAd(
    int pos, bool isCompulsory, Function? onAdLoadedCallback) {
  if (getAppLovinInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  AppLovinInterstitialAd.load(
      adUnitId: _settings?.appLovin?.interestialId ?? "",
      adLoadCallback: AppLovinInterstitialAdLoadCallback(
        onAdLoaded: (AppLovinInterstitialAd ad) {
          print('$ad loaded');
          setAppLovinInterestitialAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setAppLovinInterestitialAd(null);
          loadIntraAds(
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createFacebookInterstitialAd(
    int pos, bool isCompulsory, Function? onAdLoadedCallback) {
  if (getFacebookInterestitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  FacebookInterstitialAd.load(
      adUnitId: _settings?.facebookAds?.interestialId ?? "",
      adLoadCallback: FacebookInterstitialAdLoadCallback(
        onAdLoaded: (FacebookInterstitialAd ad) {
          print('$ad loaded');
          setFacebookInterestitialAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setFacebookInterestitialAd(null);
          loadIntraAds(
              pos: pos + 1,
              isCompulsory: isCompulsory,
              onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  print('RewardedAd _createRewardedAd Start.');

  RewardedAd.load(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      request: _request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('RewardedAd loaded.');
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdmanagerRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  print('RewardedAd _createAdmanagerRewardedAd Start.');
  resetTimerDelayed();
  RewardedAd.loadWithAdManagerAdRequest(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      adManagerRequest: _ad_manager_request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('RewardedAd loaded.');
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAppLovinRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getAppLovinRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  print('RewardedAd _createAppLovinRewardedAd Start.');

  AppLovinRewardedAd.load(
      adUnitId: _settings?.appLovin?.rewaredId ?? "",
      rewardedAdLoadCallback: AppLovinRewardedAdLoadCallback(
        onAdLoaded: (AppLovinRewardedAd ad) {
          print('RewardedAd loaded.');
          setAppLovinRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setAppLovinRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createFacebookRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getFacebookRewardedAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  print('RewardedAd _createFacebookRewardedAd Start.');
  resetTimerDelayed();
  FacebookRewardedAd.load(
      adUnitId: _settings?.facebookAds?.rewardedId ?? "",
      rewardedAdLoadCallback: FacebookRewardedAdLoadCallback(
        onAdLoaded: (FacebookRewardedAd ad) {
          print('RewardedAd loaded.');
          setFacebookRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setFacebookRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdManagerInterstitialAd(
    int pos, bool isCompulsory, Function? onAdLoadedCallback) {
  if (getAdManagerInterstitialAd() != null) {
    setTimerDelayed();
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  resetTimerDelayed();
  AdManagerInterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _ad_manager_request,
      adLoadCallback: AdManagerInterstitialAdLoadCallback(
          onAdLoaded: (AdManagerInterstitialAd ad) {
        print('$ad loaded');
        setAdManagerInterstitialAd(ad);
        getAdManagerInterstitialAd()?.setImmersiveMode(true);
        if (onAdLoadedCallback != null) {
          onAdLoadedCallback();
        }
      }, onAdFailedToLoad: (LoadAdError error) {
        print('InterstitialAd failed to load: $error.');
        setAdManagerInterstitialAd(null);
        loadIntraAds(
            pos: pos + 1,
            isCompulsory: isCompulsory,
            onAdLoadedCallback: onAdLoadedCallback);
      }));
}

void _showInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getInterestitialAd() == null) {
    _showAdManagerInterstitialAd(callback);
    return;
  }

  getInterestitialAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (InterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (InterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      incrementCounter();
      setInterestitialAd(null);
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent: (InterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getInterestitialAd()?.show();
  setInterestitialAd(null);
}

void _showAdManagerInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAdManagerInterstitialAd() == null) {
    _showAppLovinInterstitialAd(callback);
    return;
  }

  getAdManagerInterstitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AdManagerInterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AdManagerInterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      incrementCounter();
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);
      setAdManagerInterstitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AdManagerInterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setAdManagerInterstitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAdManagerInterstitialAd()?.show();
  setAdManagerInterstitialAd(null);
}

void _showAppLovinInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinInterestitialAd() == null) {
    _showFacebookInterstitialAd(callback);
    return;
  }

  getAppLovinInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinInterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AppLovinInterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      incrementCounter();
      setAppLovinInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AppLovinInterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setAppLovinInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAppLovinInterestitialAd()?.show();
  setAppLovinInterestitialAd(null);
}

void _showFacebookInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getFacebookInterestitialAd() == null) {
    if (callback != null) {
      callback();
    }
    incrementCounter();
    loadIntraAds();
    return;
  }

  getFacebookInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (FacebookInterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (FacebookInterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      incrementCounter();
      setFacebookInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (FacebookInterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      setFacebookInterestitialAd(null);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getFacebookInterestitialAd()?.show();
  setFacebookInterestitialAd(null);
}

void _showRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getRewardedAd() == null) {
    _showAppLovinRewardedAd(callback, onFailedCallback, onSkippedCallback);
    return;
  }

  getRewardedAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (RewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (RewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (RewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );

  getRewardedAd()?.setImmersiveMode(true);
  setRewardShowCompleted(false);
  getRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
    setRewardShowCompleted(true);
  });
  setRewardedAd(null);
}

void _showAppLovinRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinRewardedAd() == null) {
    _showFacebookRewardedAd(callback, onFailedCallback, onSkippedCallback);
    return;
  }

  getAppLovinRewardedAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinRewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AppLovinRewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (AppLovinRewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );

  setRewardShowCompleted(false);
  getAppLovinRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
    setRewardShowCompleted(true);
  });

  setAppLovinRewardedAd(null);
}

void _showFacebookRewardedAd(Function? callback, Function? onFailedCallback,
    Function? onSkippedCallback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getFacebookRewardedAd() == null) {
    if (onFailedCallback != null) {
      onFailedCallback();
    }
    return;
  }

  getFacebookRewardedAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (FacebookRewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (FacebookRewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      } else {
        if (onSkippedCallback != null) {
          onSkippedCallback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (FacebookRewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (onFailedCallback != null) {
        onFailedCallback();
      }
      loadRewardAds();
    },
  );

  setRewardShowCompleted(false);
  getFacebookRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
    setRewardShowCompleted(true);
  });

  setFacebookRewardedAd(null);
}